//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#include <vcl\printers.hpp>
#pragma hdrstop

#include "LineGraph.h"
//---------------------------------------------------------------------------
__fastcall LineGraph::LineGraph(TComponent* Owner)
	: TCustomControl(Owner)
{
	//   
	FNumXTicks = 5;
	FNumYTicks = 5;
	FXStart = 0.0;
	FYStart = 0.0;
	FXInc = 10.0;
	FYInc = 10.0;

	FNumPoints = 0;
	FNumberOfLines = 0;

	//    
	//      
	FXGrid = true;
	FYGrid = true;

	FXNumDecimals = 2;
	FYNumDecimals = 2;
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::SetNumberOfLines( int nLines )
{
	//   
	//  X   Y
	FXPoints.reserve( nLines );
	FYPoints.reserve( nLines );

	//   
	FLineColors.reserve( nLines );

	//    
	//   
	for ( int i=0; i<nLines; ++i )
		FLineColors[i] = clBlack;

	//   
	FNumberOfLines = nLines;
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::SetNumberOfPoints( int nPoints )
{
	//   
	//  X   Y   
	for ( int i=0; i<FNumberOfLines; ++i )
	{
		FXPoints[i].reserve( nPoints );
		FYPoints[i].reserve( nPoints );

		//    
		//     0.0
		for ( int nPt=0; nPt<nPoints; ++nPt )
		{
			FXPoints[i][nPt] = 0.0;
			FYPoints[i][nPt] = 0.0;
		}
	}

	//   
	FNumPoints = nPoints;
    FNumPoints = nPoints;
}
//---------------------------------------------------------------------------
int __fastcall LineGraph::XPointToScreen(double pt)
{
	int rightMargin = 40;
	int nXStart = 50;

	//   
	int nSpaceTicks = (Width-nXStart-rightMargin)
											/ FNumXTicks;
	int nNumPixels = nSpaceTicks * FNumXTicks;

	//   
	double dWidth = (FNumXTicks * FXInc) - FXStart;

	// ,     
	double dPercent = (pt-FXStart) / dWidth;

	//     
	int nX = dPercent * nNumPixels;

	// !     
	nX = nXStart + nX;

	return nX;
}
//---------------------------------------------------------------------------
int __fastcall LineGraph::YPointToScreen(double pt)
{
	int nYStart = Top + 20;

	//   
	int nSpaceTicks = (Height-30-nYStart) / FNumYTicks;
	int nNumPixels = nSpaceTicks * FNumYTicks;

	//   
	double dHeight = (FNumYTicks * FYInc) - FYStart;

	// ,     
	double dPercent = (pt-FYStart) / dHeight;

	//     
	int nY = dPercent * nNumPixels;

	// !     
	nY = nYStart + nY;

	return nY;
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::DoPaint(TCanvas *pCanvas)
{

	int nYStart = Top + 20;
	int nXStart = 50;
	int RightMargin = 40;

	//   
	pCanvas->MoveTo( nXStart, nYStart);
	pCanvas->LineTo( nXStart, Height-30 );
	pCanvas->LineTo( Width-RightMargin, Height-30);

	//    
	//  

	if (FNumXTicks > 0)
	{
		//  
		int nSpaceTicks = (Width-nXStart-RightMargin)
											/ FNumXTicks;
		double xVal = FXStart;

		for ( int i=0; i<=FNumXTicks; ++i )
		{
			pCanvas->MoveTo( nXStart+(i*nSpaceTicks),Height-30);
			pCanvas->LineTo( nXStart+(i*nSpaceTicks),Height-25);

			//  
			char szBuffer[ 20 ];
			sprintf(szBuffer, "%5.*lf",FXNumDecimals,xVal);

			//    ...
			int nWidth = pCanvas->TextWidth( szBuffer );

			//      
			pCanvas->Brush->Color = Color;
			pCanvas->TextOut(nXStart+(i*nSpaceTicks)-nWidth/2,
											Height-20, szBuffer );

			//  
			xVal += FXInc;

			//   ,  
			if ( FXGrid )
			{
				pCanvas->MoveTo( nXStart+(i*nSpaceTicks),
												nYStart );
				pCanvas->LineTo( nXStart+(i*nSpaceTicks),
												Height-30 );
			}

		}
	}

	//  
	if (FNumYTicks > 0)
	{
		double yVal = FYStart;

		//  
		int nSpaceTicks = (Height-30-nYStart) / FNumYTicks;

		for ( int i=0; i<=FNumYTicks; ++i )
		{
			int nYPos = Height-30-(i*nSpaceTicks);
			pCanvas->MoveTo( nXStart-5, nYPos );
			pCanvas->LineTo( nXStart, nYPos );

			//  
			char szBuffer[ 20 ];
			sprintf(szBuffer, "%5.*lf",FYNumDecimals,yVal);

			//    ...
			int nWidth = pCanvas->TextWidth( szBuffer );
			int nHeight = pCanvas->TextHeight( szBuffer );

			//      
			pCanvas->Brush->Color = Color;
			pCanvas->TextOut(nXStart-nWidth-7,
								nYPos-nHeight/2, szBuffer );

			//  
			yVal += FXInc;

			//   ,  
			if ( FYGrid )
			{
				pCanvas->MoveTo( nXStart, nYPos );
				pCanvas->LineTo( Width-RightMargin, nYPos );
			}
		}
	}

	//  ,  
	if ( FNumPoints > 0 )
	{
		for ( int nLine = 0; nLine < FXPoints.size(); ++nLine )
		{
			//     
			pCanvas->Pen->Color = FLineColors[ nLine ];

			//    
			int nXPos = XPointToScreen(FXPoints[nLine][0]);
			int nYPos = YPointToScreen(FYPoints[nLine][0]);
			for ( int i=1; i<NumberOfPoints; ++i )
			{
				pCanvas->MoveTo(nXPos, nYPos);
				nXPos = XPointToScreen(FXPoints[nLine][i]);
				nYPos = YPointToScreen(FYPoints[nLine][i]);
				pCanvas->LineTo(nXPos, nYPos);
			}
		}

		//  
		pCanvas->Pen->Color = clBlack;

	}
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::Paint(void)
{
   DoPaint(Canvas);
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::Print(void)
{
   TPrinter *printer;
   printer = new TPrinter();
   printer->BeginDoc();
   DoPaint(printer->Canvas);
   printer->EndDoc();
   delete printer;
}
//---------------------------------------------------------------------------
double __fastcall LineGraph::GetXPoint( int nLine, int nIndex )
{
   return FXPoints[ nLine][ nIndex ];
}

void __fastcall LineGraph::SetXPoint( int nLine, int nIndex, double dPoint )
{
   if ( nLine >= FXPoints.size() )
      FXPoints.insert( FXPoints.end(), DblArray() );

   FXPoints[ nLine ].insert(FXPoints[nLine].end(),dPoint);

}
//---------------------------------------------------------------------------
double __fastcall LineGraph::GetYPoint( int nLine, int nIndex )
{
   return FYPoints[ nLine ][ nIndex ];
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::SetYPoint( int nLine, int nIndex, double dPoint )
{
   if ( nLine >= FYPoints.size() )
      FYPoints.insert( FYPoints.end(), DblArray() );

   FYPoints[ nLine ].insert(FYPoints[nLine].end(),dPoint);
}
//---------------------------------------------------------------------------
void __fastcall LineGraph::SetLineColor( int nIndex, Graphics::TColor clrNewColor )
{
   FLineColors[ nIndex ] = clrNewColor;
}
//---------------------------------------------------------------------------
Graphics::TColor __fastcall LineGraph::GetLineColor( int nIndex )
{
   //   
   return FLineColors[ nIndex ];
}

//---------------------------------------------------------------------------
namespace Linegraph
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(LineGraph)};
		RegisterComponents("Graphics", classes, 0);
	}
}
//---------------------------------------------------------------------------
 
