//---------------------------------------------------------------------------
#ifndef LineGraphH
#define LineGraphH
//---------------------------------------------------------------------------
#include <vcl\sysutils.hpp>
#include <vcl\controls.hpp>
#include <vcl\classes.hpp>
#include <vcl\forms.hpp>
#include <vector>

//---------------------------------------------------------------------------
const int MaxPoints = 100;
const int MaxLines = 10;
typedef std::vector<double> DblArray;

class LineGraph : public TCustomControl
{
private:
    int FNumberOfLines;
	int FNumXTicks;
	int FNumYTicks;
    double FXStart;
    double FYStart;
    double FXInc;
    double FYInc;
    int    FNumPoints;
    std::vector<DblArray> FXPoints;
    std::vector<DblArray> FYPoints;
    std::vector<Graphics::TColor> FLineColors;
    bool   FXGrid;
    bool   FYGrid;
    int    FXNumDecimals;
    int    FYNumDecimals;
protected:
	virtual void __fastcall DoPaint(TCanvas *Canvas);
public:
	virtual __fastcall LineGraph(TComponent* Owner);
	virtual void __fastcall Paint(void);
	virtual void __fastcall Print(void);
   virtual double __fastcall GetXPoint( int nLine, int nIndex );
   virtual void __fastcall SetXPoint( int nLine, int nIndex, double dPoint );
   virtual double __fastcall GetYPoint( int nLine, int nIndex );
   virtual void __fastcall SetYPoint( int nLine, int nIndex, double dPoint );
	int __fastcall XPointToScreen(double pt);
	int __fastcall YPointToScreen(double pt);
   virtual void __fastcall SetLineColor( int nIndex, Graphics::TColor clrNewColor );
   virtual void __fastcall SetNumberOfLines( int nLines );
   virtual void __fastcall SetNumberOfPoints( int nPoints );
   virtual Graphics::TColor __fastcall GetLineColor( int nIndex );

    __property Graphics::TColor LineColors[ int nIndex ] = { read=GetLineColor, write=SetLineColor };
    __property double XPoint[int nLine][int Index] = {read=GetXPoint, write=SetXPoint};
    __property double YPoint[int nLine][int Index] = {read=GetYPoint, write=SetYPoint};

__published:
    __property int NumberXTicks = { read=FNumXTicks, write=FNumXTicks, default=5 };
    __property int NumberYTicks = { read=FNumYTicks, write=FNumYTicks, default=5 };
    __property double XStart = { read=FXStart, write=FXStart};
    __property double YStart = { read=FYStart, write=FYStart};
    __property double XIncrement = { read=FXInc, write=FXInc};
    __property double YIncrement = { read=FYInc, write=FYInc};
    __property int NumberOfPoints = { read=FNumPoints, write=SetNumberOfPoints, default=0 };
    __property bool XGrid = { read=FXGrid, write=FXGrid, default=true };
    __property bool YGrid = { read=FYGrid, write=FYGrid, default=true };
    __property int NumXDecimals = { read=FXNumDecimals, write=FXNumDecimals, default=2 };
    __property int NumYDecimals = { read=FYNumDecimals, write=FYNumDecimals, default=2 };
    __property int NumberOfLines = { read=FNumberOfLines, write=SetNumberOfLines, default=0 };
};
//---------------------------------------------------------------------------
#endif
 
